/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
import { Lifecycle, Messages, SfError, SfProject } from '@salesforce/core';
import { Duration } from '@salesforce/kit';
import { ComponentSetBuilder, RequestStatus, } from '@salesforce/source-deploy-retrieve';
import { ensure, ensureString } from '@salesforce/ts-types';
import { Flags, loglevel, requiredOrgFlagWithDeprecations, Ux } from '@salesforce/sf-plugins-core';
import { resolveZipFileName, SourceCommand } from '../../../sourceCommand.js';
import { Stash } from '../../../stash.js';
import { RetrieveResultFormatter, } from '../../../formatters/mdapi/retrieveResultFormatter.js';
Messages.importMessagesDirectoryFromMetaUrl(import.meta.url);
const messages = Messages.loadMessages('@salesforce/plugin-source', 'md.retrieve');
const spinnerMessages = Messages.loadMessages('@salesforce/plugin-source', 'spinner');
const retrieveMessages = Messages.loadMessages('@salesforce/plugin-source', 'retrieve');
const replacement = 'project retrieve start';
export class Retrieve extends SourceCommand {
    static state = 'deprecated';
    static deprecationOptions = {
        to: replacement,
        message: messages.getMessage('deprecation', [replacement]),
    };
    static hidden = true;
    static summary = messages.getMessage('retrieve.summary');
    static description = messages.getMessage('retrieve.description');
    static examples = messages.getMessages('retrieve.examples');
    static flags = {
        loglevel,
        'target-org': requiredOrgFlagWithDeprecations,
        retrievetargetdir: Flags.directory({
            char: 'r',
            summary: messages.getMessage('flags.retrievetargetdir.summary'),
            required: true,
        }),
        unpackaged: Flags.file({
            char: 'k',
            summary: messages.getMessage('flags.unpackaged.summary'),
            exclusive: ['sourcedir', 'packagenames'],
        }),
        sourcedir: Flags.directory({
            char: 'd',
            summary: messages.getMessage('flags.sourcedir.summary'),
            exclusive: ['unpackaged', 'packagenames'],
        }),
        packagenames: Flags.string({
            multiple: true,
            delimiter: ',',
            char: 'p',
            summary: messages.getMessage('flags.packagenames.summary'),
            exclusive: ['sourcedir', 'unpackaged'],
        }),
        singlepackage: Flags.boolean({
            char: 's',
            description: messages.getMessage('flags.singlepackage.description'),
            summary: messages.getMessage('flags.singlepackage.summary'),
        }),
        zipfilename: Flags.string({
            char: 'n',
            summary: messages.getMessage('flags.zipfilename.summary'),
        }),
        unzip: Flags.boolean({
            char: 'z',
            summary: messages.getMessage('flags.unzip.summary'),
        }),
        wait: Flags.duration({
            char: 'w',
            unit: 'minutes',
            summary: messages.getMessage('flags.wait.summary'),
            default: Duration.minutes(1440), // 24 hours is a reasonable default versus -1 (no timeout)
        }),
        apiversion: Flags.string({
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore force char override for backward compat
            char: 'a',
            description: messages.getMessage('flags.apiversion.description'),
            summary: messages.getMessage('flags.apiversion.summary'),
        }),
        verbose: Flags.boolean({
            summary: messages.getMessage('flags.verbose.summary'),
        }),
    };
    retrieveResult;
    sourceDir;
    retrieveTargetDir;
    zipFileName;
    unzip;
    // will be set to `flags.wait` (which has a default value) when executed.
    wait;
    isAsync;
    mdapiRetrieve;
    flags;
    org;
    async run() {
        this.flags = (await this.parse(Retrieve)).flags;
        this.retrieveTargetDir = this.resolveOutputDir(this.flags.retrievetargetdir);
        this.org = this.flags['target-org'];
        await this.retrieve();
        this.resolveSuccess();
        return this.formatResult();
    }
    async retrieve() {
        const packagenames = this.flags.packagenames;
        if (packagenames === this.unzip && !this.flags.unpackaged) {
            this.sourceDir = this.resolveRootDir(this.flags.sourcedir);
        }
        this.retrieveTargetDir = this.resolveOutputDir(this.flags.retrievetargetdir);
        const manifest = this.resolveManifest(this.flags.unpackaged);
        const singlePackage = this.flags.singlepackage;
        this.zipFileName = resolveZipFileName(this.flags.zipfilename);
        this.unzip = this.flags.unzip;
        const waitFlag = this.flags.wait;
        this.wait = waitFlag.minutes === -1 ? Duration.days(7) : waitFlag;
        this.isAsync = this.wait.quantity === 0;
        if (singlePackage && packagenames?.length) {
            throw new SfError(messages.getMessage('InvalidPackageNames', [packagenames.toString()]), 'InvalidPackageNames');
        }
        this.spinner.start(spinnerMessages.getMessage('retrieve.main', [this.org?.getUsername()]));
        this.spinner.status = spinnerMessages.getMessage('retrieve.componentSetBuild');
        this.componentSet = await ComponentSetBuilder.build({
            // use the apiVersion if provided.
            // Manifests default to their specified apiVersion(ComponentSetBuilder handles this)
            // and not specifying the apiVersion will use the max for the org/Connection
            apiversion: this.flags.apiversion,
            packagenames,
            sourcepath: this.sourceDir ? [this.sourceDir] : undefined,
            manifest: manifest
                ? {
                    manifestPath: manifest,
                    directoryPaths: [],
                }
                : undefined,
        });
        await Lifecycle.getInstance().emit('preretrieve', { packageXmlPath: manifest });
        const username = this.org?.getUsername() ?? '';
        // eslint-disable-next-line @typescript-eslint/require-await
        Lifecycle.getInstance().on('apiVersionRetrieve', async (apiData) => {
            this.log(retrieveMessages.getMessage('apiVersionMsgDetailed', [
                'Retrieving',
                apiData.manifestVersion,
                username,
                apiData.apiVersion,
            ]));
        });
        this.spinner.status = spinnerMessages.getMessage('retrieve.sendingRequest');
        this.mdapiRetrieve = await this.componentSet.retrieve({
            usernameOrConnection: username,
            output: this.retrieveTargetDir,
            packageOptions: this.flags.packagenames,
            format: 'metadata',
            singlePackage,
            zipFileName: this.zipFileName,
            unzip: this.unzip,
        });
        Stash.set('MDAPI_RETRIEVE', {
            jobid: this.mdapiRetrieve.id ?? '',
            retrievetargetdir: this.retrieveTargetDir,
            zipfilename: this.zipFileName,
            unzip: this.unzip,
        });
        this.log(`Retrieve ID: ${this.mdapiRetrieve.id ?? ''}`);
        if (this.isAsync) {
            this.spinner.stop('queued');
        }
        else {
            this.spinner.status = spinnerMessages.getMessage('retrieve.polling');
            this.retrieveResult = await this.mdapiRetrieve.pollStatus({
                frequency: Duration.milliseconds(1000),
                timeout: this.wait,
            });
            this.spinner.stop();
        }
    }
    resolveSuccess() {
        const StatusCodeMap = new Map([
            [RequestStatus.Succeeded, 0],
            [RequestStatus.Canceled, 1],
            [RequestStatus.Failed, 1],
            [RequestStatus.InProgress, 69],
            [RequestStatus.Pending, 69],
            [RequestStatus.Canceling, 69],
        ]);
        if (!this.isAsync) {
            this.setExitCode(StatusCodeMap.get(this.retrieveResult?.response.status) ?? 1);
        }
    }
    formatResult() {
        // async result
        if (this.isAsync) {
            const targetUsername = this.flags['target-org'].getUsername();
            const cmdFlags = `--jobid ${ensureString(this.mdapiRetrieve?.id)} --retrievetargetdir ${this.retrieveTargetDir}${targetUsername ? ` --targetusername ${targetUsername}` : ''}`;
            this.log('');
            this.log(messages.getMessage('checkStatus', [cmdFlags]));
            return {
                done: false,
                id: this.mdapiRetrieve?.id ?? '',
                state: 'Queued',
                status: 'Queued',
                timedOut: true,
            };
        }
        else {
            const formatterOptions = {
                waitTime: this.wait.quantity,
                verbose: this.flags.verbose ?? false,
                retrieveTargetDir: this.retrieveTargetDir ?? '',
                zipFileName: this.zipFileName,
                unzip: this.unzip,
            };
            const formatter = new RetrieveResultFormatter(new Ux({ jsonEnabled: this.jsonEnabled() }), formatterOptions, ensure(this.retrieveResult));
            if (!this.jsonEnabled()) {
                formatter.display();
            }
            return formatter.getJson();
        }
    }
    resolveProjectPath() {
        try {
            return SfProject.getInstance().getDefaultPackage().fullPath;
        }
        catch (error) {
            this.debug('No SFDX project found for default package directory');
        }
        return '';
    }
    resolveRootDir(rootDir) {
        return rootDir
            ? this.ensureFlagPath({
                flagName: 'sourcedir',
                path: rootDir,
                type: 'dir',
                throwOnENOENT: true,
            })
            : this.resolveProjectPath();
    }
    resolveOutputDir(outputDir) {
        return this.ensureFlagPath({
            flagName: 'retrievetargetdir',
            path: outputDir,
            type: 'dir',
        });
    }
    resolveManifest(manifestPath) {
        if (manifestPath?.length) {
            return this.ensureFlagPath({
                flagName: 'unpackaged',
                path: manifestPath,
                type: 'file',
                throwOnENOENT: true,
            });
        }
    }
}
//# sourceMappingURL=retrieve.js.map